/***************************************************************************
    NWNX System - System functions plugin
    Copyright (C) 2006 Ingmar Stieger (Papillon, papillon@nwnx.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ***************************************************************************/

/*

	---------------------------------------------------------------------------------------
	Date		Name		Reason
	---------------------------------------------------------------------------------------
    
    18 Jan 07   Edward      Initial release.
    05 Aug 07   Edward      Refactored to NWNX4 1.08.
    25 Aug 07   Edward      Support for multiple instances of NWN2 server.
    25 Aug 07   Edward      Added more useful error messages.
    08 Dec 07   Edward      Added a more robust server reset function.
    12 Nov 08   Edward      Recompiled for NWN2 patch 1.13 and NWNX 1.09.
    06 Apr 09   Edward      Added NWNX4 1.09 folder support.
                            CPU Core configuration is now found relative to the plugin's folder.
    07 Apr 09   Edward      Added CPU usage function.

	---------------------------------------------------------------------------------------

*/

// Definitions.
#if !defined(SYSTEM_H_INCLUDED)
#define SYSTEM_H_INCLUDED

#define DLLEXPORT extern "C" __declspec(dllexport)
#define SYSTEM_VERSION "2.1.0.0"

// Pragmatics.
#pragma warning( disable: 4995 )	// Win32 Shell Library API functions are used safely.

// Includes.
#include "windows.h"
#include "../plugin.h"
#include "../../misc/log.h"
#include "wx/tokenzr.h"
#include "wx/hashset.h"
#include "strsafe.h"
#include "wx/fileconf.h"
#include "tlhelp32.h"
#include "shlwapi.h"
#include "pdh.h"

// Classes.
class System : public Plugin{

public:

	System( );
	~System( );

	bool Init( PCHAR nwnxhome );

	INT GetInt( PCHAR psFunction, PCHAR psParam1, INT nParam2 ){ return( 0 ); }
	VOID SetInt( PCHAR psFunction, PCHAR psParam1, INT nParam2, INT nValue ){ };
	FLOAT GetFloat( PCHAR psFunction, PCHAR psParam1, INT nParam2 ){ return( 0.0 ); }
	VOID SetFloat( PCHAR psFunction, PCHAR psParam1, INT nParam2, FLOAT fValue ){ };
	
	VOID SetString( PCHAR psFunction, PCHAR psParam1, INT nParam2, PCHAR psValue );
	PCHAR GetString( PCHAR psFunction, PCHAR psParam1, INT nParam2 );

	VOID GetFunctionClass( TCHAR * fClass );

    // This method will write text to NWN2's NWNXGetString(..) return value.
	VOID WriteNWN2String( PCHAR pszNWN2String );	

    // This method will close the NW2Server, NWNX4 can automatically restart it then.
    VOID ResetServer( );

    // This method will gets the CPU usage as a percentage.
    VOID GetCpuUsage( );


private:

	wxLogNWNX * logger;
	wxFileConfig * config;

	ULONG                   uCPUID;         // CPU Core.
	PROCESSENTRY32          ProcessInfo;	// NW2Server's process info.
	HANDLE                  hProcess;		// NW2Server's process handle.

    PDH_FMT_COUNTERVALUE    pdhValue;       // Performance helper value.

};

#endif